@@ -14,6 +14,7 @@
 #include <botan/internal/pcurves_mul.h>
 #include <botan/internal/pcurves_util.h>
 #include <botan/internal/stl_util.h>
+#include <concepts>
 #include <vector>
 
 namespace Botan {
@@ -40,8 +41,6 @@ inversions. See pcurves_secp256r1.cpp or pcurves_secp256k1.cpp for examples with
 the bells and whistles.
 */
 
-namespace {
-
 /**
 * Montomgomery Representation of Integers
 *
@@ -93,7 +92,7 @@ class MontgomeryRep final {
       * Convert an integer into Montgomery representation
       */
       constexpr static std::array<W, N> to_rep(const std::array<W, N>& x) {
-         std::array<W, 2 * N> z;
+         std::array<W, 2 * N> z;  // NOLINT(*-member-init)
          comba_mul<N>(z.data(), x.data(), R2.data());
          return Self::redc(z);
       }
@@ -106,7 +105,7 @@ class MontgomeryRep final {
       */
       constexpr static std::array<W, N> wide_to_rep(const std::array<W, 2 * N>& x) {
          auto redc_x = Self::redc(x);
-         std::array<W, 2 * N> z;
+         std::array<W, 2 * N> z;  // NOLINT(*-member-init)
          comba_mul<N>(z.data(), redc_x.data(), R3.data());
          return Self::redc(z);
       }
@@ -163,6 +162,7 @@ class IntMod final {
       IntMod(Self&& other) = default;
       IntMod& operator=(const Self& other) = default;
       IntMod& operator=(Self&& other) = default;
+      ~IntMod() = default;
 
       /**
       * Return integer zero
@@ -221,18 +221,35 @@ class IntMod final {
          return !CT::Choice::from_int(v[0] & 0x01);
       }
 
+      /**
+      * Return either this or -this depending on which is even
+      */
+      constexpr Self correct_sign(CT::Choice even) const {
+         const auto flip = (even != this->is_even());
+         return Self::choose(flip, this->negate(), *this);
+      }
+
+      /**
+      * Return x or y depending on if choice is set or not
+      */
+      static constexpr Self choose(CT::Choice choice, const Self& x, const Self& y) {
+         auto r = y;
+         r.conditional_assign(choice, x);
+         return r;
+      }
+
       /**
       * Modular addition; return c = a + b
       */
       friend constexpr Self operator+(const Self& a, const Self& b) {
-         std::array<W, N> t;
+         std::array<W, N> t;  // NOLINT(*-member-init)
 
          W carry = 0;
          for(size_t i = 0; i != N; ++i) {
             t[i] = word_add(a.m_val[i], b.m_val[i], &carry);
          }
 
-         std::array<W, N> r;
+         std::array<W, N> r;  // NOLINT(*-member-init)
          bigint_monty_maybe_sub<N>(r.data(), carry, t.data(), P.data());
          return Self(r);
       }
@@ -241,13 +258,13 @@ class IntMod final {
       * Modular subtraction; return c = a - b
       */
       friend constexpr Self operator-(const Self& a, const Self& b) {
-         std::array<W, N> r;
+         std::array<W, N> r;  // NOLINT(*-member-init)
          W carry = 0;
          for(size_t i = 0; i != N; ++i) {
             r[i] = word_sub(a.m_val[i], b.m_val[i], &carry);
          }
 
-         bigint_cnd_add(carry, r.data(), N, P.data(), N);
+         bigint_cnd_add(carry, r.data(), P.data(), N);
          return Self(r);
       }
 
@@ -265,7 +282,7 @@ class IntMod final {
          W borrow = shift_right<1>(t);
 
          // If value was odd, add (P/2)+1
-         bigint_cnd_add(borrow, t.data(), N, INV_2.data(), N);
+         bigint_cnd_add(borrow, t.data(), INV_2.data(), N);
 
          return Self(t);
       }
@@ -275,7 +292,7 @@ class IntMod final {
          std::array<W, N> t = value();
          W carry = shift_left<1>(t);
 
-         std::array<W, N> r;
+         std::array<W, N> r;  // NOLINT(*-member-init)
          bigint_monty_maybe_sub<N>(r.data(), carry, t.data(), P.data());
          return Self(r);
       }
@@ -293,7 +310,7 @@ class IntMod final {
       * Modular multiplication; return c = a * b
       */
       friend constexpr Self operator*(const Self& a, const Self& b) {
-         std::array<W, 2 * N> z;
+         std::array<W, 2 * N> z;  // NOLINT(*-member-init)
          comba_mul<N>(z.data(), a.data(), b.data());
          return Self(Rep::redc(z));
       }
@@ -302,7 +319,7 @@ class IntMod final {
       * Modular multiplication; set this to this * other
       */
       constexpr Self& operator*=(const Self& other) {
-         std::array<W, 2 * N> z;
+         std::array<W, 2 * N> z;  // NOLINT(*-member-init)
          comba_mul<N>(z.data(), data(), other.data());
          m_val = Rep::redc(z);
          return (*this);
@@ -317,7 +334,7 @@ class IntMod final {
          const W mask = CT::Mask<W>::from_choice(cond).value();
 
          for(size_t i = 0; i != N; ++i) {
-            m_val[i] = choose(mask, nx.m_val[i], m_val[i]);
+            m_val[i] = Botan::choose(mask, nx.m_val[i], m_val[i]);
          }
       }
 
@@ -330,8 +347,8 @@ class IntMod final {
          const W mask = CT::Mask<W>::from_choice(cond).value();
 
          for(size_t i = 0; i != N; ++i) {
-            x.m_val[i] = choose(mask, nx.m_val[i], x.m_val[i]);
-            y.m_val[i] = choose(mask, ny.m_val[i], y.m_val[i]);
+            x.m_val[i] = Botan::choose(mask, nx.m_val[i], x.m_val[i]);
+            y.m_val[i] = Botan::choose(mask, ny.m_val[i], y.m_val[i]);
          }
       }
 
@@ -345,9 +362,25 @@ class IntMod final {
          const W mask = CT::Mask<W>::from_choice(cond).value();
 
          for(size_t i = 0; i != N; ++i) {
-            x.m_val[i] = choose(mask, nx.m_val[i], x.m_val[i]);
-            y.m_val[i] = choose(mask, ny.m_val[i], y.m_val[i]);
-            z.m_val[i] = choose(mask, nz.m_val[i], z.m_val[i]);
+            x.m_val[i] = Botan::choose(mask, nx.m_val[i], x.m_val[i]);
+            y.m_val[i] = Botan::choose(mask, ny.m_val[i], y.m_val[i]);
+            z.m_val[i] = Botan::choose(mask, nz.m_val[i], z.m_val[i]);
+         }
+      }
+
+      /**
+      * Conditional swap
+      *
+      * If `cond` is true, swaps the values of `x` and `y`
+      */
+      static constexpr void conditional_swap(CT::Choice cond, Self& x, Self& y) {
+         const W mask = CT::Mask<W>::from_choice(cond).value();
+
+         for(size_t i = 0; i != N; ++i) {
+            auto nx = Botan::choose(mask, y.m_val[i], x.m_val[i]);
+            auto ny = Botan::choose(mask, x.m_val[i], y.m_val[i]);
+            x.m_val[i] = nx;
+            y.m_val[i] = ny;
          }
       }
 
@@ -357,7 +390,7 @@ class IntMod final {
       * Returns the square of this after modular reduction
       */
       constexpr Self square() const {
-         std::array<W, 2 * N> z;
+         std::array<W, 2 * N> z;  // NOLINT(*-member-init)
          comba_sqr<N>(z.data(), this->data());
          return Self(Rep::redc(z));
       }
@@ -370,7 +403,7 @@ class IntMod final {
       * (Alternate view, returns this raised to the 2^nth power)
       */
       constexpr void square_n(size_t n) {
-         std::array<W, 2 * N> z;
+         std::array<W, 2 * N> z;  // NOLINT(*-member-init)
          for(size_t i = 0; i != n; ++i) {
             comba_sqr<N>(z.data(), this->data());
             m_val = Rep::redc(z);
@@ -385,7 +418,7 @@ class IntMod final {
       constexpr Self negate() const {
          const W x_is_zero = ~CT::all_zeros(this->data(), N).value();
 
-         std::array<W, N> r;
+         std::array<W, N> r;  // NOLINT(*-member-init)
          W carry = 0;
          for(size_t i = 0; i != N; ++i) {
             r[i] = word_sub(P[i] & x_is_zero, m_val[i], &carry);
@@ -424,6 +457,7 @@ class IntMod final {
          tbl[0] = (*this);
 
          for(size_t i = 1; i != WindowElements; ++i) {
+            // Conditional ok: table indexes are public here
             if(i % 2 == 1) {
                tbl[i] = tbl[i / 2].square();
             } else {
@@ -435,6 +469,7 @@ class IntMod final {
 
          const size_t w0 = read_window_bits<WindowBits>(std::span{exp}, (Windows - 1) * WindowBits);
 
+         // Conditional ok: this function is variable time
          if(w0 > 0) {
             r = tbl[w0 - 1];
          }
@@ -444,6 +479,7 @@ class IntMod final {
 
             const size_t w = read_window_bits<WindowBits>(std::span{exp}, (Windows - i - 1) * WindowBits);
 
+            // Conditional ok: this function is variable time
             if(w > 0) {
                r *= tbl[w - 1];
             }
@@ -477,13 +513,15 @@ class IntMod final {
       static constexpr void _invert_vartime_div2_helper(Self& a, Self& x) {
          constexpr auto INV_2 = p_div_2_plus_1(Rep::P);
 
+         // Conditional ok: this function is variable time
          while((a.m_val[0] & 1) != 1) {
             shift_right<1>(a.m_val);
 
             W borrow = shift_right<1>(x.m_val);
 
+            // Conditional ok: this function is variable time
             if(borrow) {
-               bigint_add2_nc(x.m_val.data(), N, INV_2.data(), N);
+               bigint_add2(x.m_val.data(), N, INV_2.data(), N);
             }
          }
       }
@@ -524,6 +562,7 @@ class IntMod final {
       * a/y resp.
       */
       constexpr Self invert_vartime() const {
+         // Conditional ok: this function is variable time
          if(this->is_zero().as_bool()) {
             return Self::zero();
          }
@@ -542,6 +581,7 @@ class IntMod final {
          Self::_invert_vartime_div2_helper(a, y);
 
          for(;;) {
+            // Conditional ok: this function is variable time
             if(a.m_val == b.m_val) {
                // At this point it should be that a == b == 1
                auto r = y.negate();
@@ -562,9 +602,10 @@ class IntMod final {
             * Compute r = b - a and check if it underflowed
             * If it did not then we are in the b > a path
             */
-            std::array<W, N> r;
+            std::array<W, N> r;  // NOLINT(*-member-init)
             word carry = bigint_sub3(r.data(), b.data(), N, a.data(), N);
 
+            // Conditional ok: this function is variable time
             if(carry == 0) {
                // b > a
                b.m_val = r;
@@ -583,9 +624,9 @@ class IntMod final {
       /**
       * Return the modular square root if it exists
       *
-      * The CT::Choice indicates if the square root exists or not.
+      * The CT::Option will be unset if the square root does not exist
       */
-      constexpr std::pair<Self, CT::Choice> sqrt() const {
+      constexpr CT::Option<Self> sqrt() const {
          if constexpr(Self::P_MOD_4 == 3) {
             // The easy case for square root is when p == 3 (mod 4)
 
@@ -595,7 +636,7 @@ class IntMod final {
             // Zero out the return value if it would otherwise be incorrect
             const CT::Choice correct = (z.square() == *this);
             z.conditional_assign(!correct, Self::zero());
-            return {z, correct};
+            return CT::Option<Self>(z, correct);
          } else {
             // Shanks-Tonelli, following I.4 in RFC 9380
 
@@ -634,7 +675,7 @@ class IntMod final {
             // Zero out the return value if it would otherwise be incorrect
             const CT::Choice correct = (z.square() == *this);
             z.conditional_assign(!correct, Self::zero());
-            return {z, correct};
+            return CT::Option<Self>(z, correct);
          }
       }
 
@@ -715,12 +756,14 @@ class IntMod final {
       * also rejected.
       */
       static std::optional<Self> deserialize(std::span<const uint8_t> bytes) {
+         // Conditional ok: input length is public
          if(bytes.size() != Self::BYTES) {
             return {};
          }
 
          const auto words = bytes_to_words<W, N, BYTES>(bytes.first<Self::BYTES>());
 
+         // Conditional acceptable: std::optional is implicitly not constant time
          if(!bigint_ct_is_lt(words.data(), N, P.data(), N).as_bool()) {
             return {};
          }
@@ -750,6 +793,7 @@ class IntMod final {
       * modular reduces it.
       */
       static constexpr std::optional<Self> from_wide_bytes_varlen(std::span<const uint8_t> bytes) {
+         // Conditional ok: input length is public
          if(bytes.size() > 2 * Self::BYTES) {
             return {};
          }
@@ -773,7 +817,7 @@ class IntMod final {
       static Self random(RandomNumberGenerator& rng) {
          constexpr size_t MAX_ATTEMPTS = 1000;
 
-         std::array<uint8_t, Self::BYTES> buf;
+         std::array<uint8_t, Self::BYTES> buf{};
 
          for(size_t i = 0; i != MAX_ATTEMPTS; ++i) {
             rng.randomize(buf);
@@ -785,6 +829,7 @@ class IntMod final {
                buf[0] &= mask;
             }
 
+            // Conditionals ok: rejection sampling reveals only values we didn't use
             if(auto s = Self::deserialize(buf)) {
                if(s.value().is_nonzero().as_bool()) {
                   return s.value();
@@ -801,7 +846,7 @@ class IntMod final {
       * Notice this function is consteval, and so can only be called at compile time
       */
       static consteval Self constant(int8_t x) {
-         std::array<W, 1> v;
+         std::array<W, 1> v{};
          v[0] = (x >= 0) ? x : -x;
          auto s = Self::from_words(v);
          return (x >= 0) ? s : s.negate();
@@ -829,17 +874,12 @@ class IntMod final {
 template <typename FieldElement, typename Params>
 class AffineCurvePoint final {
    public:
-      // We can't pass a FieldElement directly because FieldElement is
-      // not "structural" due to having private members, so instead
-      // recreate it here from the words.
-      static constexpr FieldElement A = FieldElement::from_words(Params::AW);
-      static constexpr FieldElement B = FieldElement::from_words(Params::BW);
-
       static constexpr size_t BYTES = 1 + 2 * FieldElement::BYTES;
-      static constexpr size_t COMPRESSED_BYTES = 1 + FieldElement::BYTES;
 
       using Self = AffineCurvePoint<FieldElement, Params>;
 
+      // Note this constructor does not check the validity of the x/y pair
+      // This must be verified prior to this constructor being called
       constexpr AffineCurvePoint(const FieldElement& x, const FieldElement& y) : m_x(x), m_y(y) {}
 
       constexpr AffineCurvePoint() : m_x(FieldElement::zero()), m_y(FieldElement::zero()) {}
@@ -847,7 +887,9 @@ class AffineCurvePoint final {
       static constexpr Self identity() { return Self(FieldElement::zero(), FieldElement::zero()); }
 
       // Helper for ct_select of pcurves_generic
-      static constexpr Self identity(const Self&) { return Self(FieldElement::zero(), FieldElement::zero()); }
+      static constexpr Self identity(const Self& /*unused*/) {
+         return Self(FieldElement::zero(), FieldElement::zero());
+      }
 
       constexpr CT::Choice is_identity() const { return x().is_zero() && y().is_zero(); }
 
@@ -855,6 +897,7 @@ class AffineCurvePoint final {
       AffineCurvePoint(Self&& other) = default;
       AffineCurvePoint& operator=(const Self& other) = default;
       AffineCurvePoint& operator=(Self&& other) = default;
+      ~AffineCurvePoint() = default;
 
       constexpr Self negate() const { return Self(x(), y().negate()); }
 
@@ -888,48 +931,6 @@ class AffineCurvePoint final {
          return result;
       }
 
-      /**
-      * Return (x^3 + A*x + B) mod p
-      */
-      static constexpr FieldElement x3_ax_b(const FieldElement& x) { return (x.square() + Self::A) * x + Self::B; }
-
-      /**
-      * Point deserialization
-      *
-      * This accepts compressed or uncompressed formats.
-      */
-      static std::optional<Self> deserialize(std::span<const uint8_t> bytes) {
-         if(bytes.size() == Self::BYTES && bytes[0] == 0x04) {
-            auto x = FieldElement::deserialize(bytes.subspan(1, FieldElement::BYTES));
-            auto y = FieldElement::deserialize(bytes.subspan(1 + FieldElement::BYTES, FieldElement::BYTES));
-
-            if(x && y) {
-               const auto lhs = (*y).square();
-               const auto rhs = Self::x3_ax_b(*x);
-               if((lhs == rhs).as_bool()) {
-                  return Self(*x, *y);
-               }
-            }
-         } else if(bytes.size() == Self::COMPRESSED_BYTES && (bytes[0] == 0x02 || bytes[0] == 0x03)) {
-            const CT::Choice y_is_even = CT::Mask<uint8_t>::is_equal(bytes[0], 0x02).as_choice();
-
-            if(auto x = FieldElement::deserialize(bytes.subspan(1, FieldElement::BYTES))) {
-               auto [y, is_square] = x3_ax_b(*x).sqrt();
-
-               if(is_square.as_bool()) {
-                  const auto flip_y = y_is_even != y.is_even();
-                  y.conditional_assign(flip_y, y.negate());
-                  return Self(*x, y);
-               }
-            }
-         } else if(bytes.size() == 1 && bytes[0] == 0x00) {
-            // See SEC1 section 2.3.4
-            return Self::identity();
-         }
-
-         return {};
-      }
-
       /**
       * Return the affine x coordinate
       */
@@ -979,11 +980,22 @@ class ProjectiveCurvePoint {
       * Convert a point from affine to projective form
       */
       static constexpr Self from_affine(const AffinePoint& pt) {
-         if(pt.is_identity().as_bool()) {
-            return Self::identity();
-         } else {
-            return ProjectiveCurvePoint(pt.x(), pt.y());
-         }
+         /*
+         * If the point is the identity element (x=0, y=0) then instead of
+         * creating (x, y, 1) = (0, 0, 1) we want our projective identity
+         * encoding of (0, 1, 0)
+         *
+         * Which we can achieve by a conditional swap of y and z if the
+         * affine point is the identity.
+         */
+
+         auto x = pt.x();
+         auto y = pt.y();
+         auto z = FieldElement::one();
+
+         FieldElement::conditional_swap(pt.is_identity(), y, z);
+
+         return ProjectiveCurvePoint(x, y, z);
       }
 
       /**
@@ -1013,6 +1025,7 @@ class ProjectiveCurvePoint {
       ProjectiveCurvePoint(Self&& other) = default;
       ProjectiveCurvePoint& operator=(const Self& other) = default;
       ProjectiveCurvePoint& operator=(Self&& other) = default;
+      ~ProjectiveCurvePoint() = default;
 
       friend constexpr Self operator+(const Self& a, const Self& b) { return Self::add(a, b); }
 
@@ -1096,6 +1109,7 @@ class ProjectiveCurvePoint {
          // In certain contexts we may be called with a Null_RNG; in that case the
          // caller is accepting that randomization will not occur
 
+         // Conditional ok: caller's RNG state (seeded vs not) is presumed public
          if(rng.is_seeded()) {
             auto r = FieldElement::random(rng);
 
@@ -1220,6 +1234,11 @@ class EllipticCurve {
          (Params::Z != 0 && A.is_nonzero().as_bool() && B.is_nonzero().as_bool() && FieldElement::P_MOD_4 == 3);
 
       static constexpr bool OrderIsLessThanField = bigint_cmp(NW.data(), Words, PW.data(), Words) == -1;
+
+      /**
+      * Return (x^3 + A*x + B) mod p
+      */
+      static constexpr FieldElement x3_ax_b(const FieldElement& x) { return (x.square() + A) * x + B; }
 };
 
 /**
@@ -1247,6 +1266,9 @@ class BlindedScalarBits final {
       //
       // This can return any value between 0 and the scalar bit length, as long
       // as it is a multiple of the word size.
+      //
+      // TODO(Botan4) this function should be consteval but cannot currently to a bug
+      // in older versions of Clang. Change to consteval when minimum Clang is bumped.
       static constexpr size_t blinding_bits(size_t sb) {
          constexpr size_t wb = WordInfo<W>::bits;
 
@@ -1294,7 +1316,7 @@ class BlindedScalarBits final {
                // bytes of the scalar together. At worst, it's equivalent to
                // omitting the blinding entirely.
 
-               std::array<uint8_t, C::Scalar::BYTES> sbytes;
+               std::array<uint8_t, C::Scalar::BYTES> sbytes = {};
                scalar.serialize_to(sbytes);
                for(size_t i = 0; i != sbytes.size(); ++i) {
                   maskb[i % mask_bytes] ^= sbytes[i];
@@ -1312,7 +1334,7 @@ class BlindedScalarBits final {
 
             // Compute masked scalar s + k*n
             comba_mul<n_words>(mask_n, mask, C::NW.data());
-            bigint_add2_nc(mask_n, 2 * n_words, sw.data(), sw.size());
+            bigint_add2(mask_n, 2 * n_words, sw.data(), sw.size());
 
             std::reverse(mask_n, mask_n + 2 * n_words);
             m_bytes = store_be<std::vector<uint8_t>>(mask_n);
@@ -1335,6 +1357,11 @@ class BlindedScalarBits final {
          CT::unpoison(m_bytes.data(), m_bytes.size());
       }
 
+      BlindedScalarBits(const BlindedScalarBits& other) = delete;
+      BlindedScalarBits(BlindedScalarBits&& other) = delete;
+      BlindedScalarBits& operator=(const BlindedScalarBits& other) = delete;
+      BlindedScalarBits& operator=(BlindedScalarBits&& other) = delete;
+
    private:
       // TODO this could be a fixed size array
       std::vector<uint8_t> m_bytes;
@@ -1345,7 +1372,7 @@ class UnblindedScalarBits final {
    public:
       static constexpr size_t Bits = C::Scalar::BITS;
 
-      UnblindedScalarBits(const typename C::Scalar& scalar) { scalar.serialize_to(std::span{m_bytes}); }
+      explicit UnblindedScalarBits(const typename C::Scalar& scalar) { scalar.serialize_to(std::span{m_bytes}); }
 
       size_t get_window(size_t offset) const {
          // Extract a WindowBits sized window out of s, depending on offset.
@@ -1370,7 +1397,8 @@ class PrecomputedBaseMulTable final {
 
       static constexpr size_t Windows = (BlindedScalar::Bits + WindowBits - 1) / WindowBits;
 
-      PrecomputedBaseMulTable(const AffinePoint& p) : m_table(basemul_setup<C, WindowBits>(p, BlindedScalar::Bits)) {}
+      explicit PrecomputedBaseMulTable(const AffinePoint& p) :
+            m_table(basemul_setup<C, WindowBits>(p, BlindedScalar::Bits)) {}
 
       ProjectivePoint mul(const Scalar& s, RandomNumberGenerator& rng) const {
          const BlindedScalar scalar(s, rng);
@@ -1405,7 +1433,7 @@ class WindowedMulTable final {
       // 2^W elements, less the identity element
       static constexpr size_t TableSize = (1 << WindowBits) - 1;
 
-      WindowedMulTable(const AffinePoint& p) : m_table(varpoint_setup<C, TableSize>(p)) {}
+      explicit WindowedMulTable(const AffinePoint& p) : m_table(varpoint_setup<C, TableSize>(p)) {}
 
       ProjectivePoint mul(const Scalar& s, RandomNumberGenerator& rng) const {
          const BlindedScalar bits(s, rng);
@@ -1459,7 +1487,7 @@ class WindowedBoothMulTable final {
       // 2^W elements [1*P, 2*P, ..., 2^W*P]
       static constexpr size_t TableSize = 1 << TableBits;
 
-      WindowedBoothMulTable(const AffinePoint& p) : m_table(varpoint_setup<C, TableSize>(p)) {}
+      explicit WindowedBoothMulTable(const AffinePoint& p) : m_table(varpoint_setup<C, TableSize>(p)) {}
 
       ProjectivePoint mul(const Scalar& s, RandomNumberGenerator& rng) const {
          const BlindedScalar bits(s, rng);
@@ -1473,6 +1501,7 @@ class WindowedBoothMulTable final {
             const size_t w_i = bits.get_window(idx);
             const auto [tidx, tneg] = booth_recode<WindowBits>(w_i);
 
+            // Conditional ok: loop iteration count is public
             if(i == 0) {
                accum = ProjectivePoint::from_affine(m_table.ct_select(tidx));
                accum.conditional_assign(tneg, accum.negate());
@@ -1482,6 +1511,7 @@ class WindowedBoothMulTable final {
 
             accum = accum.dbl_n(WindowBits);
 
+            // Conditional ok: loop iteration count is public
             if(i <= 3) {
                accum.randomize_rep(rng);
             }
@@ -1497,7 +1527,7 @@ class WindowedBoothMulTable final {
       }
 
    private:
-      template <size_t B, typename T>
+      template <size_t B, std::unsigned_integral T>
       static constexpr std::pair<size_t, CT::Choice> booth_recode(T x) {
          static_assert(B < sizeof(T) * 8 - 2, "Invalid B");
 
@@ -1572,6 +1602,7 @@ class VartimeMul2Table final {
          bool s1_is_zero = s1.is_zero().as_bool();
          bool s2_is_zero = s2.is_zero().as_bool();
 
+         // Conditional ok: this function is variable time
          if(s1_is_zero && s2_is_zero) {
             return ProjectivePoint::identity();
          }
@@ -1581,6 +1612,7 @@ class VartimeMul2Table final {
                const size_t w_1 = bits1.get_window((Windows - i - 1) * WindowBits);
                const size_t w_2 = bits2.get_window((Windows - i - 1) * WindowBits);
                const size_t window = w_1 + (w_2 << WindowBits);
+               // Conditional ok: this function is variable time
                if(window > 0) {
                   return std::make_pair(window, i);
                }
@@ -1600,6 +1632,7 @@ class VartimeMul2Table final {
 
             const size_t window = w_1 + (w_2 << WindowBits);
 
+            // Conditional ok: this function is variable time
             if(window > 0) {
                accum += m_table[window - 1];
             }
@@ -1654,23 +1687,18 @@ inline auto map_to_curve_sswu(const typename C::FieldElement& u) -> typename C::
    const auto tv1 = invert_field_element<C>(z2_u4 + z_u2);
    auto x1 = SSWU_C1<C>() * (C::FieldElement::one() + tv1);
    x1.conditional_assign(tv1.is_zero(), SSWU_C2<C>());
-   const auto gx1 = C::AffinePoint::x3_ax_b(x1);
-
    const auto x2 = z_u2 * x1;
-   const auto gx2 = C::AffinePoint::x3_ax_b(x2);
 
-   // Will be zero if gx1 is not a square
-   const auto [gx1_sqrt, gx1_is_square] = gx1.sqrt();
-
-   auto x = x2;
    // By design one of gx1 and gx2 must be a quadratic residue
-   auto y = gx2.sqrt().first;
+   const CT::Option<typename C::FieldElement> y1 = sqrt_field_element<C>(C::x3_ax_b(x1));
+   const CT::Option<typename C::FieldElement> y2 = sqrt_field_element<C>(C::x3_ax_b(x2));
 
-   C::FieldElement::conditional_assign(x, y, gx1_is_square, x1, gx1_sqrt);
+   const auto use_y1 = y1.has_value();
 
-   y.conditional_assign(y.is_even() != u.is_even(), y.negate());
+   auto x = C::FieldElement::choose(use_y1, x1, x2);
+   auto y = C::FieldElement::choose(use_y1, y1.value_or(C::FieldElement::zero()), y2.value_or(C::FieldElement::zero()));
 
-   auto pt = typename C::AffinePoint(x, y);
+   auto pt = typename C::AffinePoint(x, y.correct_sign(u.is_even()));
 
    CT::unpoison(pt);
    return pt;
@@ -1689,13 +1717,13 @@ inline auto map_to_curve_sswu(const typename C::FieldElement& u) -> typename C::
 */
 template <typename C, bool RO, std::invocable<std::span<uint8_t>> ExpandMsg>
    requires C::ValidForSswuHash
-inline auto hash_to_curve_sswu(ExpandMsg expand_message)
+inline auto hash_to_curve_sswu(const ExpandMsg& expand_message)
    -> std::conditional_t<RO, typename C::ProjectivePoint, typename C::AffinePoint> {
    constexpr size_t SecurityLevel = (C::OrderBits + 1) / 2;
    constexpr size_t L = (C::PrimeFieldBits + SecurityLevel + 7) / 8;
    constexpr size_t Cnt = RO ? 2 : 1;
 
-   std::array<uint8_t, L * Cnt> uniform_bytes;
+   std::array<uint8_t, L * Cnt> uniform_bytes = {};
    expand_message(uniform_bytes);
 
    if constexpr(RO) {
@@ -1711,8 +1739,6 @@ inline auto hash_to_curve_sswu(ExpandMsg expand_message)
    }
 }
 
-}  // namespace
-
 }  // namespace Botan
 
 #endif
