@@ -366,7 +366,7 @@ std::pair<DilithiumSeedRho, DilithiumPolyVec> decode_public_key(StrongSpan<const
  * NIST FIPS 204, Algorithm 24 (skEncode)
  */
 DilithiumSerializedPrivateKey encode_keypair(const DilithiumInternalKeypair& keypair) {
-   auto& [pk, sk] = keypair;
+   const auto& [pk, sk] = keypair;
    BOTAN_ASSERT_NONNULL(pk);
    BOTAN_ASSERT_NONNULL(sk);
    const auto& mode = sk->mode();
@@ -501,7 +501,7 @@ std::optional<std::tuple<DilithiumCommitmentHash, DilithiumPolyVec, DilithiumPol
    for(auto& p : response) {
       poly_unpack_gamma1(p, slicer, mode);
    }
-   BOTAN_ASSERT_NOMSG(slicer.remaining() == mode.omega() + mode.k());
+   BOTAN_ASSERT_NOMSG(slicer.remaining() == size_t(mode.omega()) + mode.k());
 
    auto hint = hint_unpack(slicer, mode);
    BOTAN_ASSERT_NOMSG(slicer.empty());
@@ -861,7 +861,7 @@ DilithiumPolyVec make_hint(const DilithiumPolyVec& z, const DilithiumPolyVec& r,
 
    for(size_t i = 0; i < r.size(); ++i) {
       for(size_t j = 0; j < r[i].size(); ++j) {
-         hint[i][j] = make_hint(z[i][j], r[i][j]).as_bool();
+         hint[i][j] = static_cast<int>(make_hint(z[i][j], r[i][j]).as_bool());
       }
    }
 
